/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | foam-extend: Open Source CFD
   \\    /   O peration     |
    \\  /    A nd           | For copyright notice see file Copyright
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of foam-extend.

    foam-extend is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.

    foam-extend is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with foam-extend.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "rotatingWallTangentialVelocityFvPatchVectorField.H"
#include "addToRunTimeSelectionTable.H"
#include "volFields.H"
#include "surfaceFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

rotatingWallTangentialVelocityFvPatchVectorField::rotatingWallTangentialVelocityFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchField<vector>(p, iF),
    origin_(vector::zero),
    axis_(vector::zero),
    omega_(0),
    axialVelocity_(0),
    radialVelocity_(0),
    tangVelocity_(0)
{}


rotatingWallTangentialVelocityFvPatchVectorField::rotatingWallTangentialVelocityFvPatchVectorField
(
    const rotatingWallTangentialVelocityFvPatchVectorField& ptf,
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchField<vector>(ptf, p, iF, mapper),
    origin_(ptf.origin_),
    axis_(ptf.axis_),
    omega_(ptf.omega_),
//////////////

    axialVelocity_(ptf.axialVelocity_),
    radialVelocity_(ptf.radialVelocity_),
    tangVelocity_(ptf.tangVelocity_)
//////////////
{}


rotatingWallTangentialVelocityFvPatchVectorField::rotatingWallTangentialVelocityFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchField<vector>(p, iF),
    origin_(dict.lookup("origin")),
    axis_(dict.lookup("axis")),
    omega_(readScalar(dict.lookup("omega"))),
///////////////////
    axialVelocity_(readScalar(dict.lookup("axialVelocity"))),
    radialVelocity_(readScalar(dict.lookup("radialVelocity"))),
    tangVelocity_(readScalar(dict.lookup("tangVelocity")))
////////////////
{
    // Evaluate the wall velocity
    updateCoeffs();
}


rotatingWallTangentialVelocityFvPatchVectorField::rotatingWallTangentialVelocityFvPatchVectorField
(
    const rotatingWallTangentialVelocityFvPatchVectorField& pivpvf
)
:
    fixedValueFvPatchField<vector>(pivpvf),
    origin_(pivpvf.origin_),
    axis_(pivpvf.axis_),
    omega_(pivpvf.omega_),
////////////////
    axialVelocity_(pivpvf.axialVelocity_),
    radialVelocity_(pivpvf.radialVelocity_),
    tangVelocity_(pivpvf.tangVelocity_)
//////////////
{}


rotatingWallTangentialVelocityFvPatchVectorField::rotatingWallTangentialVelocityFvPatchVectorField
(
    const rotatingWallTangentialVelocityFvPatchVectorField& pivpvf,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchField<vector>(pivpvf, iF),
    origin_(pivpvf.origin_),
    axis_(pivpvf.axis_),
    omega_(pivpvf.omega_),
///////////////////////

    axialVelocity_(pivpvf.axialVelocity_),
    radialVelocity_(pivpvf.radialVelocity_),
    tangVelocity_(pivpvf.tangVelocity_)
///////////

{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void rotatingWallTangentialVelocityFvPatchVectorField::updateCoeffs()
{
    if (updated())
    {
        return;
    }

//    const scalar t = this->db().time().timeOutputValue();
//    const scalar axialVelocity = axialVelocity_->value(t);
//    const scalar radialVelocity = radialVelocity_->value(t);
    // Calculate the rotating wall velocity from the specification of the motion
   // vectorField Up = (-omega_)*((patch().Cf() - origin_) ^ (axis_/mag(axis_)));

    // Remove the component of Up normal to the wall
    // just in case it is not exactly circular
//    vectorField n = patch().nf();
//    vectorField::operator=(Up - n*(n & Up));


    vector hatAxis = axis_/mag(axis_);

    const vectorField r(patch().Cf() - origin_);
    const vectorField d(r - (hatAxis & r)*hatAxis);

    //tmp<vectorField> tangDir
    vectorField tangDir
    (
          (patch().Cf() - origin_) ^ (axis_/mag(axis_)) 
    );

    operator==(-omega_*tangDir  + -tangVelocity_*tangDir/mag(tangDir) + hatAxis*axialVelocity_ + radialVelocity_*d/mag(d));
    fixedValueFvPatchVectorField::updateCoeffs();
}


void rotatingWallTangentialVelocityFvPatchVectorField::write(Ostream& os) const
{
    fvPatchVectorField::write(os);
    os.writeKeyword("origin") << origin_ << token::END_STATEMENT << nl;
    os.writeKeyword("axis") << axis_ << token::END_STATEMENT << nl;
    os.writeKeyword("omega") << omega_ << token::END_STATEMENT << nl;
    os.writeKeyword("axialVelocity") << axialVelocity_ << token::END_STATEMENT << nl;
    os.writeKeyword("radialVelocity") << radialVelocity_ << token::END_STATEMENT << nl;
    os.writeKeyword("tangVelocity") << tangVelocity_ << token::END_STATEMENT << nl;
    writeEntry("value", os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField
(
    fvPatchVectorField,
    rotatingWallTangentialVelocityFvPatchVectorField
);


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
